/*
 *  print.c - code to handle printing a file.
 */

#include <stdio.h>
#include <string.h>
#include <windows.h>

// Note that everybody's print.h has the wrong type for the
// third argument to OpenJob(), that's why I have to cast
// the HDC to an HPJOB.

#define PRINTDRIVER /* to enable definitions in print.h */
#include <print.h>


#define BUFFER_SIZE 4096

int     PrintFile(HDC PrinterContext, char *Port,
                          HWND ParentWindow, char *Filename)
    {
    HPJOB   Job;
    char   *Buffer = new char[BUFFER_SIZE+1];

    if(Buffer == NULL)
        {
        MessageBox(ParentWindow,
                        "Insufficient memory for file copy",
                             "PassThru", MB_OK|MB_ICONSTOP);
        return FALSE;
        }
    FILE   *InputFile = fopen(Filename, "rb");
    if(InputFile == NULL)
        {
        char    Message[128];
        sprintf(Message,
             "Can't open file '%s' for reading.", Filename);
        MessageBox(ParentWindow, Message, "PassThru",
                                         MB_OK|MB_ICONSTOP);
        delete Buffer;
        return FALSE;
        }

    Job = OpenJob(Port, Filename, (HPJOB)PrinterContext);
    StartSpoolPage(Job);

    HCURSOR OldCursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
    size_t NBytes = 0;
    for(;;)
        {
        NBytes  = fread(Buffer, 1, BUFFER_SIZE, InputFile);
        if(NBytes > 0)
            WriteSpool(Job, Buffer, NBytes);
        else
            break;
        }
    SetCursor(OldCursor);
    EndSpoolPage(Job);
    CloseJob(Job);
    DeleteDC(PrinterContext);
    fclose(InputFile);
    delete Buffer;
    return TRUE;
    }
